// import 'express-async-errors';
import express,{Router, Request, Response } from 'express';
import {Task} from '../models/Task';
import {myDataSource} from '../app-data-source';
import { User } from '../models/User';
import { Member, MemberRole } from '../models/Member';
import { Equal, In, Not } from 'typeorm';
import { DateTimeTask } from '../models/DateTimeTask';
import { auth, authAuthorOrExecutorOfTask, subtractHours, taskFinderById } from '../helpers';
import { Project } from '../models/Project';
const router:Router = express.Router();
const dataSource = myDataSource;


/**get all tasks */
router.get('/', async(req:Request, res:Response):Promise<Response> => {
    try{
    const tasks = await dataSource
.getRepository(Task)
.find({ 
    relations:{
        executor:true,
        author:true,
    }})
    return res.send({tasks}) 
} catch(e){
    return res.status(502).send({message:(e as Error).message})
}
})


/**create new task for table MyTask*/
router.post('/', auth, async(req:Request, res:Response):Promise<Response>=>{
    try{
    const {user,title,description,project,executor,dateTimeStart,dateTimeDue, dateTimeDeadLine,priority, calendar} = req.body;
    let dateTimeDueFinal = dateTimeDue 
    let dateTimeStartFinal = dateTimeStart
    let dateTimeDeadlineFinal= dateTimeDeadLine
    if (!dateTimeDue && dateTimeDeadLine) { 
        dateTimeDueFinal = dateTimeDeadLine
        dateTimeStartFinal = subtractHours(dateTimeDueFinal, 1)
    }

    const newTask = new Task();
    newTask.title = title;
    newTask.description = description;
    if (project) {

        const watcherOfProject = await dataSource
        .getRepository(Project)
        .findOne({
            relations:{
                members:true,
        }, 
        where:
            {
            id:project.id,
            members:
            {
                user:{id:user.id},
                roleProject:MemberRole.WATCHER,
            },  
        }
    })
    if (watcherOfProject) return res.send({message:"watcher could nt make tasks"})
    newTask.project = project;
} else {
    const personalProject = await dataSource
    .getRepository(Project)
    .findOne({
        where:{
            members:{
                user:{id:user.id},
            },
            title:"Личные дела"
        }
    })
    if(personalProject){
        newTask.project = personalProject;
    } else {
        
        const newPersonalProject = new Project();
        newPersonalProject.title="Личные дела";
        newPersonalProject.color="green";
        await newPersonalProject.save()
        newTask.project = newPersonalProject;
        const newMember = new Member();
        newMember.user= user;
        newMember.project=newPersonalProject;
        newMember.roleProject = MemberRole.ADMIN;
        await newMember.save();
    }
}
    newTask.dateTimeDeadLine= dateTimeDeadlineFinal;
    newTask.author= user;
    if(newTask.project.title==="Личные дела"){
        newTask.executor=user;
    } else {
        newTask.executor= executor;
    }
    newTask.priority = priority;
    await newTask.save(); 
    if (calendar) {
    const newDateTimeTask = new DateTimeTask();
        newDateTimeTask.dateTimeStart = dateTimeStartFinal
        newDateTimeTask.dateTimeDue = dateTimeDueFinal
        newDateTimeTask.task = newTask
        await newDateTimeTask.save()
    }
    return res.send({newTask});
} catch(e){
    return res.status(502).send({message:(e as Error).message})
}

})


//** create new task for table UsersTasks*/
router.post('/users', auth, async(req:Request, res:Response):Promise<Response>=>{
    try{
    const {user,title,description,project,executor,dateTimeStart,dateTimeDue, dateTimeDeadLine,priority} = req.body;
    const newTask = new Task();
    newTask.title = title;
    newTask.description = description;
    const currentTime = new Date();
    if (project) {
        const watcherOfProject = await dataSource
        .getRepository(Project)
        .findOne({
            relations:{
                members:true,
        }, 
        where:
            {
            id:project.id,
            members:
            {
                user:{id:user.id},
                roleProject:MemberRole.WATCHER,
            },  
        }
    })
    if (watcherOfProject) return res.send({message:"watcher could nt make tasks"})
    newTask.project = project;
} else {
    const personalProject = await dataSource
    .getRepository(Project)
    .findOne({
        where:{
            members:{
                user:{id:user.id},
            },
            title:"Не определено"
        }
    })
    if(personalProject){
        newTask.project = personalProject;
    } else {
        const newPersonalProject = new Project();
        newPersonalProject.title="Не определено";
        newPersonalProject.color="green";
        await newPersonalProject.save()
        newTask.project = newPersonalProject;
        const newMember = new Member();
        newMember.user= user;
        newMember.project=newPersonalProject;
        newMember.roleProject = MemberRole.ADMIN;
        await newMember.save();
    }
}
    
    newTask.dateTimeDeadLine=dateTimeDeadLine;
    newTask.author= user;
    newTask.executor= executor;
    newTask.priority = priority;
    await newTask.save(); 
    if(dateTimeDue && dateTimeStart){
        const newDateTimeTask = new DateTimeTask();
        newDateTimeTask.dateTimeStart = dateTimeStart
        newDateTimeTask.dateTimeDue = dateTimeDue
        newDateTimeTask.task = newTask
        await newDateTimeTask.save()
    }
    return res.send({newTask});

} catch(e){
    return res.status(502).send({message:(e as Error).message})
}
})


/**check tasks of specific user by userID */
router.get('/user/:userId', async (req: Request, res: Response):Promise<Response>=>{   
    try{
    const userId = req.params.userId;
    const tasks = await dataSource
    .getRepository(Task)
    .find({
        relations:{
            executor:true,
        },
    where:
        {
        executor:{
            id:userId
        }
    },
})
    return res.send({tasks})
} catch(e){
    return res.status(502).send({message:(e as Error).message})
}
})


/**check tasks of current user where he is  executor(works for tbl MyTasks), search by id*/
router.get('/my',auth, async (req: Request, res: Response):Promise<Response>=>{
    let projectsRawId =[]
    try{
    const user = req.body.user
    projectsRawId = await dataSource
    .getRepository(Project)
    .find({
        select:{
            id:true
        },
            where:{
                title:Not(Equal("Не определено")),
                members:{
                    user:{
                        id: user.id
                    },
                    roleProject:Not(MemberRole.WATCHER)
                }
            }
        }
    )
    
    const projectsList= projectsRawId.map(project=> {return project.id}) 
    const tasks = await dataSource
    .getRepository(Task)
    .find(
        {relations:{
            author:true,
            executor:true,
            project:true
        },
        where:{
            executor:{
                    id:user.id
                },
            project:{
                id:In(projectsList)
            }
        }
    }
    )
    const projects = await dataSource
    .getRepository(Project)
    .find({
        relations:{
            members:true
        },
        where:{
            id:In(projectsList)
        }
    })
        return res.send({tasks, projects}) 
    } catch(e){
        return res.status(502).send({message:(e as Error).message})
    }
    })


/**return tasks & users of projects in which current user is involved, search by token*/
router.get('/related', auth,async (req: Request, res: Response):Promise<Response>=>{
    try{
    const user = req.body.user
    const tasks = await dataSource
    .createQueryBuilder()
    .select(["task"])
    .from(Task,"task")
    .leftJoinAndSelect("task.executor","user")
    .leftJoinAndSelect("task.dateTimeTasks","dateTimeTask")
    .leftJoinAndSelect("task.author","users")
    .leftJoinAndSelect("task.project","project")
    .leftJoinAndSelect("project.members", "member")
    .where("member.userId = :userId", {userId:user.id})
    .getMany()

     let projectIds:any[] = []
    if (tasks.length>0){
        for (let task of tasks){
            const projectId = task?.project?.id
            projectIds.push(projectId)
        }
    }
    const projectsIdArray = [...new Set(projectIds)]; 
    let users:User[]=[]
    if (projectIds.length>0){
    users = await dataSource
   .getRepository(User)
.find({
        relations:{
        members:true,
        },
        where:{
        members:
            {project:In(
                projectsIdArray
            )}
        }
    })
}
    return res.send({tasks, users})
} catch(e){
    return res.status(502).send({message:(e as Error).message})
}
})


/**delete of task by task id */
router.delete('/:taskId',async (req: Request, res: Response):Promise<Response>=>{
    try{
    const taskId = req.params.taskId;
    await myDataSource
    .createQueryBuilder()
    .delete()
    .from(Task)
    .where("id = :id", { id: taskId })
    .execute()
    return res.send({message: 'Task deleted successfully'})
} catch(e){
    return res.status(502).send({message:(e as Error).message})
}

})


/**change of task  by task id */
router.put('/:taskId',auth,authAuthorOrExecutorOfTask,async(req:Request, res:Response):Promise<Response>=> {
    try{
    const {taskId} = req.params 
    const {authorStatus,executorStatus,title,description,note, archive,project,dateTimeTaskId,dateTimeStart,
        dateTimeDue,executor,accomplish,dateTimeDeadLine, dateTimeFactDeadLine,priority} = req.body;
            
    const task = await taskFinderById(taskId)
    if (!task) return res.status(404).send({Message:'task not found'})
    let dateTimeTask = null;
    if (dateTimeTaskId) {
        const dateTimeTaskData = await dataSource
        .createQueryBuilder()
        .select("dateTimeTask")
        .from(DateTimeTask, "dateTimeTask")
        .where("dateTimeTask,id=:dateTimeTaskId",{dateTimeTaskId})
        .getOne()
        if (!dateTimeTask) return res.status(404).send({Message:'dateTimeTask not found'})
        dateTimeTask = dateTimeTaskData
    }
        
        if (dateTimeTask!==null) {
            dateTimeTask.dateTimeStart = dateTimeStart
            dateTimeTask.dateTimeDue = dateTimeDue
            await dateTimeTask.save()
    }   
    
    if (authorStatus){
        task.title = title
        task.description= description
        task.archive=archive
        task.dateTimeDeadLine=dateTimeDeadLine;
        task.project=project
        task.executor=executor
        task.priority= priority   
    }
    if(executorStatus){
        task.note = note
        task.dateTimeFactDeadLine= dateTimeFactDeadLine
    }
    task.accomplish= accomplish
    await task.save()
    return res.send({message:'update task successfully'})
    
} catch(e){
    return res.status(502).send({message:(e as Error).message})
}
})


/** search all tasks on given projects: project Id array[] */
router.post('/project',async (req: Request, res: Response):Promise<Response>=>{
    let projectArray :string[]= req.body;
    try{
    if (projectArray.length===0) {
        const rawTasks = await
        dataSource
       .getRepository(Task)
        .find({
            relations:{
                executor:true,
                author:true,
                project:true
            }
        })
    const tasks:object[]= rawTasks.filter(task=>task.project===null)
       return res.send({tasks})
    } 
    const tasks = await
     dataSource
    .getRepository(Task)
    
    .createQueryBuilder('task')
    .innerJoinAndSelect( 'task.project', 'project')
    .innerJoinAndSelect('task.author', 'users')
    .innerJoinAndSelect('task.executor', 'userss')
    .innerJoinAndSelect( 'task.dateTimeTasks', 'DateTimeTask')
    .where('task.project IN(:...projects)', {projects:projectArray})
    .getMany()
    

    return res.send({tasks})
} catch(e){
    return res.status(502).send({message:(e as Error).message})
}
}) 



/**check other users tasks that are related to the user through common projects, by user token*/
router.get('/users-tasks', auth,async (req: Request, res: Response):Promise<Response>=>{
        const {user} = req.body
        try{
            const projectsWithTasks = await dataSource
            .getRepository(Project)
            .find(
                {   relations:{
                    members:true,
                    tasks:{
                        author:true, 
                        executor:true
                    }
                },
                    where:{
                members:{
                    roleProject:Not(Equal(MemberRole.WATCHER)),
                    user:{
                        id:user.id 
                    },
                    project:{
                        title:Not(Equal("Личные дела"))
                    }
                }
            }})

            if (!projectsWithTasks) return res.status(404).send({Message:'projects not found'})
            let tasks:Task[] = []
            if (projectsWithTasks.length>0) {
                for (let project of projectsWithTasks){
                    if (project?.tasks) {
                        tasks=tasks.concat(project.tasks)
                    }
                }
            }

            const projectIds= projectsWithTasks.map(project=> {return project.id})
            const projects = await dataSource
            .getRepository(Project)
            .find({
                relations:{
                    members:{
                        user:true
                    }},
                    where:{
                        id:In(projectIds),
                        members:{
                            roleProject: Not(MemberRole.WATCHER)
                        }
                    },
                })
            return res.send({projects, tasks})
        } catch(e){
            console.log('Error: ',(e as Error).message);
            return res.status(502).send({message:(e as Error).message})
        }
})


export default router;
